local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")
local faction = UnitFactionGroup("player")

local idEnchantingVellum = 38682

local partsMemory = { }
local resourcesMemory = { }

function Capitalist:PartMemoryReset()
	partsMemory = { }
end
function Capitalist:ResourceMemoryReset()
	resourcesMemory = { }
end

function Capitalist:IsComponent(componentName)
	if (self.db.profile.components and
		self.db.profile.components[componentName]) then

		return true
	end
	return false
end

function Capitalist:IsPart(componentName)
	return (self:IsComponent(componentName) and
			self:IsProduct(componentName))
end
function Capitalist:IsResource(componentName)
	return (self:IsComponent(componentName) and
			(not self:IsProduct(componentName)))
end

function Capitalist:GetComponentId(componentName)
	if (componentName == L["Enchanting Vellum"]) then
		return idEnchantingVellum
	else
		return self.db.profile.components[componentName].id
	end
end

function Capitalist:ResourceGetMerchant(resourceName)
	if (self.db.profile.components and
		self.db.profile.components[resourceName]) then
		return self.db.profile.components[resourceName].merchant
	end
	return nil
end

function Capitalist:ResourceSetMerchant(resourceName, merchant)
	self.db.profile.components[resourceName].merchant = merchant
end

function Capitalist:OutputComponent(prefix, itemName, count)
	self:Print(self:StringForItem(prefix, self:GetComponentText(itemName), count).." @ "..self:PrettyCurrency(self:GetResourceCopperCost(itemName)).." ea.")
end

function Capitalist:OutputMissingCosts()
	local resources = {}
	for resourceName, resourceData in pairs(resourcesMemory) do
		table.insert(resources, resourceName)
	end

	local outputtedHeaderCost

	-- TODO: skip the go-between resources table (now that resourcesMemory is available, vs. componentsMemory)

	i = 1
	while (i <= #resources) do
		local resourceName = resources[i]

		if (not self.db.profile.components[resourceName].cost) then
			if (not outputtedHeaderCost) then
				self:Print(L["Resources missing cost information:"])
				outputtedHeaderCost = true
			end
			self:Print(string.format("  "..self:GetComponentText(resourceName)))
		end

		i = i + 1
	end

	if (not outputtedHeaderCost) then
		self:Print(L["All resource buy prices and costs set."])
	end
end

function Capitalist:GetComponentText(itemName)
	if (self.db.profile.components and
		self.db.profile.components[itemName] and
		self.db.profile.components[itemName].link) then

		return self.db.profile.components[itemName].link
	end
	
	return itemName
end

function Capitalist:PartIsVisible(partName)
	return partsMemory[partName] ~= nil
end
function Capitalist:PartSetVisible(profession, partName, visible)
	if (visible) then
		if (not partsMemory[partName]) then
			partsMemory[partName] = { }
		end
	else
		partsMemory[partName] = nil
	end
end

function Capitalist:ResourceIsVisible(componentName, profession)
	local isVisible = (resourcesMemory[componentName] ~= nil)
	if (profession) then
		isVisible = (resourcesMemory[componentName].professions[profession] ~= nil)
	end
	return isVisible
end
function Capitalist:ResourceSetVisible(profession, componentName, visible)
	if (visible) then
		if (not resourcesMemory[componentName]) then
			resourcesMemory[componentName] = { }
		end
		if (not resourcesMemory[componentName].professions) then
			resourcesMemory[componentName].professions = { }
		end
		-- REVIEW: When gems become parts (due to salvaging) this logic needs to automatically apply to that instead
		if (not resourcesMemory[componentName].professions[profession]) then
			resourcesMemory[componentName].professions[profession] = { }
			--print(profession)
			--if (profession == L["Jewelcrafting"]) then
				--print(componentName)
			--end
		end
	else
		resourcesMemory[componentName] = nil
	end
end

function Capitalist:GetVisibleResources(includeSalvage)
	local resourceList = {}
	for resourceName, resourceData in pairs(resourcesMemory) do
		table.insert(resourceList, resourceName)
	end

	if (includeSalvage) then
		-- salvage source materials
		local oRealm = self:GetProfile_Realm_Faction(realm, faction)
		if (oRealm) then
			for crafter, characterData in pairs(oRealm) do
				if ((character == crafter) or
					self.db.profile.optionsSpecific[realm][character].crossShop) then

					for characterSection, characterSectionData in pairs(characterData) do
						if (not self.db.profile.professions[characterSection]) then

							-- REVIEW: This excludes disenchant from shopping...
							-- Including them might be useful but results in pages and pages
							-- of disenchantables.  Maybe if there was some way to bucket them
							-- in some way by disenchant expectations/ratios/itemlevel+type/whatever.
							if (characterSection ~= L["Disenchant"]) then
								for skillTarget, skillTargetData in pairs(characterSectionData) do
									if ((skillTarget ~= "recipeLink") and
										(skillTarget ~= "numConsume")) then
										-- Assume non-recipeLink, non-numConsume entries are skillTarget sections
					
										local skillTargetName = self:LinkExtractName(skillTarget)
										if (skillTargetName) then
											if (not tContains(resourceList, skillTargetName)) then
--print("GVR - "..skillTargetName)
												table.insert(resourceList, skillTargetName)
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end
	end

	return resourceList
end

function Capitalist:GetVisibleComponents()
	local componentList = self:GetVisibleResources(true)

	for partName, partData in pairs(partsMemory) do
		table.insert(componentList, partName)
	end

	return componentList
end

function Capitalist:GetSortedVisibleComponents()
	local componentList = self:GetVisibleComponents()
	table.sort(componentList, function(a, b) return a < b end)
	return componentList
end

function Capitalist:StringForItem(prefix, item, count)
	local s = item;
	if (prefix) then
		s = prefix..s
	end
	if (count and (count > 1)) then
		s = s.." ["..count.."]"
	end
	return s
end
